package uhk.salay.npl.demonpl.watson;

import java.util.*;
import javax.servlet.http.HttpServletRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import uhk.salay.npl.demonpl.jpa.model.*;
import uhk.salay.npl.demonpl.jpa.repositorys.RelationRepository;

@Controller
@RequestMapping(value = "/ser")
public class SearchController {
    private final static Logger logger = LogManager.getLogger(SearchController.class);

    @Autowired
    private WatsonService service;
    @Autowired
    private RelationRepository relationRepository;


    @GetMapping(value = "*")
    public String get(Model model) {
        model.addAttribute("searchContent", new PostContent());
        return "search";
    }

    @GetMapping(value = "/entities")
    public String getEntities(Model model) {
        model.addAttribute("searchContent", new PostContent());
        return "searchForEntity";
    }

    @PostMapping(value = "/entities")
    public String postEntities(@ModelAttribute("searchContent") PostContent searchContent, Model model) {
        final Collection<Entity> entities = service.getEntities(searchContent.getText());
        model.addAttribute("entities", entities);
        return "searchForEntity";
    }

    @GetMapping(value = "/entitiesLocal")
    public String getEntitiesLocal(Model model) {
        model.addAttribute("searchContent", new PostContent());
        return "searchForEntityLocal";
    }

    @PostMapping(value = "/entitiesLocal")
    public String postEntitiesLocal(@ModelAttribute("searchContent") PostContent searchContent, Model model) {
        Collection<Entity> entities = new ArrayList<>();
        String text = searchContent.getText();
        for (String name : text.split(";")) {
            entities.addAll(service.getEntitiesLocal(name));
        }
        model.addAttribute("entities", entities);
        return "searchForEntityLocal";
    }

    @GetMapping(value = "/rel")
    public String getRel(Model model) {
        model.addAttribute("searchContent", new PostContent());
        return "searchForRelation";
    }

    @PostMapping(value = "/rel")
    public String postRelations(@ModelAttribute("searchContent") PostContent searchContent,
                                @ModelAttribute("sButton") String button,
                                Model model, HttpServletRequest request) {

        List<Relation> relations = service.getRelationsFor(searchContent.getEntityForRelation(),
                searchContent.getTypeForRelation());
        model.addAttribute("relations", relations);


        return "searchForRelation";
    }

    @GetMapping(value = "/file")
    public String getFile(Model model) {
        model.addAttribute("searchContent", new PostContent());
        return "File";
    }

    @PostMapping(value = "/file")
    public String postFile(@ModelAttribute("searchContent") PostContent searchContent, Model model, HttpServletRequest request
//                           ,@ModelAttribute("checkBoxEmotion") boolean emotion
//                           ,@ModelAttribute("checkBoxContext") boolean context
    ) {
        List<File> files = new ArrayList<>();
        if (searchContent.isAllBooks()) {
            files = service.getAllFile();
        } else {
            String text = searchContent.getText();
            if (!text.isEmpty()) {
                files = service.getFileByPath(text);
            }
        }
        files.forEach(file -> {
            file.setCategoriesSet(file.getCategories());
            file.setConceptSet(file.getConcept());
            file.getEmotions();
            file.setKeywordSet(file.getKeyword());
        });
        model.addAttribute("files", files);
        model.addAttribute("searchContent", searchContent);
        return "File";
    }

    @GetMapping(value = "/context")
    public String getContext(Model model) {
        model.addAttribute("searchContent", new PostContent());
        return "searchForContext";
    }

    @PostMapping(value = "/context")
    public String postContext(@ModelAttribute("searchContent") PostContent searchContent, Model model,
                              @ModelAttribute("sButton") String button,
                              HttpServletRequest request) {
        if (button != null) {
            switch (button) {
                case "file":
                    service.getFileByContext(searchContent.getText(), false);
                    break;
                case "parts":
                    service.getFileByContext(searchContent.getText(), true);
                    break;
                default:
                    break;
            }
        }
        return "searchForContext";
    }

    @GetMapping(value = "/uploadFile")
    public String getLoadData(Model model) {
        model.addAttribute("searchContent", new PostContent());
        model.addAttribute("features", WatsonFeatures.values());
        return "uploadFile";
    }

    @PostMapping(value = "/uploadFile")
    public String postLoadData(@ModelAttribute("searchContent") PostContent loadContent, Model model,
                               @ModelAttribute("sButton") String button,
                               @ModelAttribute("chx_entities") boolean entities,
                               @ModelAttribute("chx_relations") boolean relation,
                               @ModelAttribute("chx_keyWordds") boolean keyWords,
                               @ModelAttribute("chx_entities_semantic") boolean entitiesSemantic,
                               @ModelAttribute("chx_sentiment") boolean sentiment,
                               @ModelAttribute("chx_concept") boolean concept,
                               @ModelAttribute("chx_categories") boolean categories
    ) throws Exception {

        final String pathToLoad = loadContent.getPathToLoad();
        Set<WatsonFeatures> features = new HashSet<>();
        if (entities) {
            features.add(WatsonFeatures.ENTITIES);
            features.add(WatsonFeatures.ENTITIES_SENTIMENT);
            features.add(WatsonFeatures.ENTITIES_EMOTION);
            features.add(WatsonFeatures.ENTITIES_MENTIONS);
        }
        if (relation) {
            features.add(WatsonFeatures.RELATIONS);
        }
        if (sentiment) {
            features.add(WatsonFeatures.SENTIMENT);
        }
        if (concept) {
            features.add(WatsonFeatures.CONCEPTS);
        }
        if (categories) {
            features.add(WatsonFeatures.CATEGORIES);
        }
        if (pathToLoad != null) {
            service.loadDataFrom(pathToLoad, features);

        }
        return "uploadFile";
    }

    @GetMapping(value = "/table")
    public String getIndex(Model model) {
        List<File> files = new ArrayList<>();
        File file = new File();
        file.setId(10);
        files.add(file);
        Part part = new Part();
        part.setId((short) 15);
        List<Part> parts = new ArrayList<>();
        parts.add(part);
        file.setParts(parts);
        model.addAttribute("searchContent", new PostContent());

//        model.addAttribute("files", files);
        return "table";
    }

}
