package uhk.salay.npl.demonpl.watson;

import com.ibm.watson.natural_language_understanding.v1.model.*;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.junit.jupiter.params.provider.ValueSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import uhk.salay.npl.demonpl.DemonplApplicationTests;
import uhk.salay.npl.demonpl.jpa.model.Entity;
import uhk.salay.npl.demonpl.jpa.model.Part;
import uhk.salay.npl.demonpl.jpa.model.Relation;
import uhk.salay.npl.demonpl.jpa.repositorys.EntityRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.FileRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.RelationRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.TypeRepository;

@PropertySource("classpath:project.properties")
public class WatsonServiceTest extends DemonplApplicationTests {

    AnalysisResults results;
    //  private String text = "IBM won Best Actor" +
//      " in a Leading Role for his performance.";
//  private String text = "NCR, which counts IBM founder Thomas Watson as one of its early employees, said its products and services account for more than $400 billion in annual commerce and 23 billion consumer serf-service transactions. The company's Philippine office established in the 1960s currently operates a research and development facility in Cebu with about 500 employees.";
    private String text = "";
    private WatsonService watsonServiceLocal;
    @Autowired
    private EntityRepository entityRepository;
    @Autowired
    private RelationRepository relationsRepository;
    @Autowired
    private TypeRepository typeRepository;
    @Autowired
    private FileRepository fileRepository;
    @Value("${modelId}")
    private String modelId;

    @Before
    public void setUp() throws Exception {
        watsonServiceLocal = new WatsonService("_KCAMVRQH8w7vTLnc4Xnyy6IKNWShoydzeoiuHrGhTo2", "https://gateway-fra.watsonplatform.net/natural-language-understanding/api");
        watsonServiceLocal.init();
//        text = FileUtils.readFileToString(new File("/home/radek/workplace/demonlp2/src/test/resources/Text.txt"), StandardCharsets.UTF_8);
//        text = FileUtils.readFileToString(new File("C:\\Worspace\\knihovna\\src\\test\\resources\\Raven.txt"), StandardCharsets.UTF_8);

//        results = getWatsonResult(text);
    }

    @Test
    public void conceptTest() throws IOException {
//                text = FileUtils.readFileToString(new File("C:\\Worspace\\knihovna\\src\\test\\resources\\Raven.txt"), StandardCharsets.UTF_8);
        results = getWatsonResult("It was normal day.");
        System.out.println();
    }

    @Test
    public void getFileByContext() {
        Set<String> part = watsonService.getFileByContext("Hobbit", false);
        Assert.assertEquals("Text.txt part: 5213", part.iterator().next());

        Set<String> file = watsonService.getFileByContext("Hobbit", true);
        Assert.assertEquals("Text.txt", file.iterator().next());
    }

    @Test
    public void procesTextOnEntities() throws Exception {
        watsonServiceLocal.setEntityRepository(entityRepository);
        watsonServiceLocal.setTypeRepository(typeRepository);
        final Collection<Entity> entityCollection = watsonServiceLocal.processTextOnEntities(results,null, null, false, false, false);
        System.out.println(Arrays.asList(entityCollection).toString());
        Assert.assertEquals(2, entityCollection.size());
    }

    @Test
    public void procesTextOnRelations() throws Exception {
        final Collection<Relation> relations = watsonServiceLocal.processTextOnRelations(results, null);
        Assert.assertEquals(1, relations.size());
    }

    @Test
    public void processTextOnKeyWords() throws Exception {
        uhk.salay.npl.demonpl.jpa.model.File file = new uhk.salay.npl.demonpl.jpa.model.File();
        file.setKeyword("");

        watsonServiceLocal.processTextOnKeyWords(results, null, file);
    }

    @ParameterizedTest
    @CsvSource({
            "--Mr. Bingley,Person"
    })
    public void getRelationsFor(String entity, String type) {
        final List<Relation> relationsFor = watsonService.getRelationsFor(entity, type);
        for (Relation relation : relationsFor) {
//            System.out.println(relation.getType() + " relation type");
//            System.out.println(relation.getEntityFrom().getName() + " entityFrom name");
//            System.out.println(relation.getEntityFrom().getType().getName() + " entityFrom type name");
//            System.out.println(relation.getEntityTo().getName() + " entityTo name");
//            System.out.println(relation.getEntityTo().getType().getName() + " entityTo type name");
            System.out.println(relation.getEntityFrom().getName()
                    + " " + relation.getType()
                    + " " + relation.getEntityTo().getName()
                    + " file " + relation.getFile().getId());
        }

    }

    @Test
    public void loadDataFrom() throws Exception {
        Set<WatsonFeatures> features = new HashSet<WatsonFeatures>() {{
            add(WatsonFeatures.ENTITIES);
            add(WatsonFeatures.RELATIONS);
            add(WatsonFeatures.KEY_WORDS);
            add(WatsonFeatures.CONCEPTS);
            add(WatsonFeatures.CATEGORIES);
            add(WatsonFeatures.ENTITIES_SENTIMENT);
            add(WatsonFeatures.ENTITIES_EMOTION);
            add(WatsonFeatures.ENTITIES_MENTIONS);
            add(WatsonFeatures.SENTIMENT);
            add(WatsonFeatures.EMOTION);
        }};
//    watsonService.loadDataFrom("/home/radek/workplace/demonlp2/src/test/resources/Text.txt", features);
//        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\alice_in_wonderland.txt", features);
//        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\Around the World in 80 Days.txt", features);
//        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\Dorothy and the Wizard in Oz.txt", features);
        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\Moby Dick.txt", features);
        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\Pride And Prejudice.txt", features);
        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\R.U.R.txt", features);
        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\Raven.txt", features);
        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\Tarzan of the Apes.txt", features);
        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\The Hobbit.txt", features);
        watsonService.loadDataFrom("C:\\Worspace\\knihovna\\src\\test\\resources\\The Three Musketeers.txt", features);

    }

    @Test
    public void getFile() {
        final Optional<uhk.salay.npl.demonpl.jpa.model.File> byId = fileRepository.findById(2);
        byId.get().getCategoriesSet();
        byId.get().getConceptSet();
        byId.get().getKeywordSet();
        System.out.println();
    }

    @Test
    public void parts() throws Exception {
        File file = FileUtils.getFile("e:\\skola\\diplomka\\knihy\\hobit\\The Hobbit byJ  RR Tolkien EBOOK_djvu.txt");
        final String contentOfFile = FileUtils.readFileToString(file, StandardCharsets.UTF_8);
        final uhk.salay.npl.demonpl.jpa.model.File fileDb = watsonService.saveFileToDb(file);

        final List<Part> parts = watsonService.cutFileToParts(fileDb, contentOfFile);
        for (Part part : parts) {
            watsonService.processConcepts(results, part, fileDb);
            watsonService.processCategories(results, part, fileDb);
        }
//    System.out.println(parts.size());
    }


    AnalysisResults getWatsonResult(String text) {
        final Features.Builder featureBuilder = new Features.Builder();

        // Entity
        EntitiesOptions entities = new EntitiesOptions.Builder()
                .sentiment(true)
                .emotion(true)
                .mentions(true)
                .build();
        featureBuilder.entities(entities);

        // relations
        RelationsOptions relations = new RelationsOptions.Builder()
                .build();
        featureBuilder.relations(relations);

        //keyWords
        KeywordsOptions keywordsOptions = new KeywordsOptions.Builder()
                .sentiment(true)
                .emotion(true)
                .build();
        featureBuilder.keywords(keywordsOptions);
        //sentiment
        SentimentOptions sentimentOptions = new SentimentOptions.Builder()
                .build();
        featureBuilder.sentiment(sentimentOptions);

        ConceptsOptions concepts = new ConceptsOptions.Builder()
                .build();
        featureBuilder.concepts(concepts);

        CategoriesOptions categoriesOptions = new CategoriesOptions.Builder()
                .explanation(true)
                .build();
        featureBuilder.categories(categoriesOptions);


        AnalyzeOptions parameters = new AnalyzeOptions.Builder()
                .text(text)
                .features(featureBuilder.build())
                .build();


        AnalysisResults response = watsonServiceLocal.getNaturalLanguageUnderstanding()
                .analyze(parameters)
                .execute()
                .getResult();

        return response;
    }


}
